/* 
HALLSTAT()
	This procedure accepts the five statistics from the Hall & Sola model (two parameters and their vcv).
	It returns the corresponding two t-statistics and the Wald statistic.
	The input should be N x 5, and the output is N x 3.
*/

proc HALLSTAT(param);

local nobs, ncol, i, b0, b1, s0, s1, s01, t0, t1, Wald01, outstats;

nobs = rows(param);
ncol = cols(param);

if fne(ncol,5);
	"Aborting: received only ";; ncol;; "columns of estimated parameters instead of 5.";
	retp(0);
else;
	"Received";; nobs;; "rows of data.";
	param = packr(missex(param,dotfeq(param[.,3],0)));
	nobs = rows(param);
	"Of these,";; nobs;; "are valid rows to be processed.";

	call dstat(0,param);
	"X1=B0, X2=B1, X3=SIGMA^2_0, X4=SIGMA^2_1, X5=SIGMA_01";
	?;?;

	outstats=zeros(nobs,3);
	for i (1,nobs,1);
		b0 = param[i,1];	
		b1 = param[i,2];	
		s0 = param[i,3];	
		s1 = param[i,4];	
		s01 = param[i,5];
		if b0 > b1;
			b0 = param[i,2];	
			b1 = param[i,1];	
			s0 = param[i,4];	
			s1 = param[i,3];	
		endif;
		outstats[i,1] = b0 / sqrt(s0);
		outstats[i,2] = b1 / sqrt(s1);
		outstats[i,3] = (b0-b1)^2 / (s0 + s1 - 2*s01);
	endfor;
endif;

retp(outstats);

endp;
