/* SHUFFLES()
	This procedure accepts an N x M matrix x and a seed.
	It randomly re-orders the rows of x and returns it.
*/
proc shuffles(x,seed);
local i,nobs,ncol;

	seed = seed;
	nobs=rows(x);
	ncol=cols(x);
	i = rndus(nobs,ncol,seed);
	x = sortc( (i~x),1);

retp(x[.,2:ncol+1]);
endp;


/* HALLSHFL()
	This procedure accepts initial parameter estimates for the NM model as well as an 
	N x 2 data matrix.  (A full Beta vector can be accepted as well.)
	It then estimates the Hall & Sola model and returns a vector of parameter estimates 
	and elements of the vcv. This vector has the format
	
		DF coefficient in regime 0
		DF coefficient in regime 1
		variance of DF coef in regime 0
		variance of DF coef in regime 1
		covariance of the two DF coeffs
*/

clearg nrows, ix;

proc HallShfl(dta,beta_NM);

local terms, beta0, f, g, h, retcode;

	ix=1; @heterscedastic model @
	if rows(beta_nm)==5;
		beta0 = beta_NM[1,1] | zeros(2,1) | beta_NM[2,1] | zeros(2,1) | 
			  -beta_NM[3,1] | beta_NM[3,1] | beta_NM[4 5,1];
	else;
		beta0 = beta_nm;
	endif;
	beta0 = em(beta0,dta);
	if rows(beta0) < 10;
		"mkvshufl: EM algorithm failed - beta vector is";;
		beta0';
		retp(zeros(5,1));
	else;
		{beta0,f,g,h,retcode} =  maxlik(dta,0,&swmkv,beta0);
		if rows(h)<10;
			"mkvshufl: maxlik() converged but vcv failed.";
			retp(zeros(5,1));
		elseif retcode > 0;
			"mkvshufl: maxlik() failed - return code is";;
			retcode;
			retp(zeros(5,1));
		else;
			retp( beta0[2,1] | beta0[4,1] | h[2,2] | h[4,4] | h[2,4]);
		endif;
	endif;

endp;


