/* SHUFFLE()
	This procedure accepts an N x M matrix x and randomly re-orders its rows.
	The re-ordered N x M matrix is returned.
*/
proc shuffle(x);
local i,nobs,ncol;

	nobs=rows(x);
	ncol=cols(x);
	i = rndu(nobs,1);
	x = sortc( (i~x),1);

retp(x[.,2:ncol+1]);
endp;


/* SCASCADE()
	This procedure accepts initial parameter estimates for the NM and VR models as well as an N x 2 data matrix.
	It then estimates the ECP model (starting from beta_VR), the NMP model (starting from beta_NM) and the SWP model.
		If either beta_VR or beta_NM is missing, it will make its own starting estimates.
	Thereafter, it returns a 5 x 1 vector of test statistics, which are
		the LR statistic testing NMP vs SWP
		the LR statistic testing EC vs SWP  (this was formerly ECP vs SWP -- this code is commented out.)
		the Wald statistic testing B_S0 = B_C0
		the Wald statistic testing B_Sb = B_Cb
		return code (sum of the return codes for ECP, NMP and SWP for maxlik().)
*/

proc SCascade(dta,beta_VR,beta_NM);

local swd,nobs, beta0, yjunk, waldstat, retpstat,xec,llfec,xecp,llfecp,xnmp,llfnmp,xswp,llfswp,g,h,retcode;

	nobs = rows(dta);
	retpstat=zeros(6,1);
	_max_GradProc=&swgrad;

@ The EC Model @
@	__title="Error Correction Model: (Constant Switching Prob.)";@
@	?;      __title;        @
	_emecm=1;
	swd = dta[.,1]~ones(nobs,1)~dta[.,2]~ones(nobs,1);
	_n1=2;  _n2=1;  _n3=1;
	if rows(beta_VR)==4;
		beta0 = beta_VR[1,1] | 0.0 | beta_VR[2:4,1];
	else;
		{beta0}=becinit(swd);
	endif;
	{xec,llfec,g,h,retcode}=swemmle(beta0,swd);
	retpstat[6,1]=retcode;

@ The ECP Model @
@	__title="Error Correction Model: (Variable Switching Prob.)";@
@	?;      __title;        @
	_emecm=1;
/*
	swd = dta[.,1]~ones(nobs,1)~dta[.,2]~ones(nobs,1)~dta[.,2]~(dta[.,2].^2);
	_n1=2;  _n2=1;  _n3=3;
	if rows(beta_VR)==4;
		beta0 = beta_VR[1,1] | 0.0 | beta_VR[2,1] | zeros(2,1) | beta_VR[3:4,1];
*/
	swd = dta[.,1]~ones(nobs,1)~dta[.,2]~ones(nobs,1)~(abs(dta[.,2])-meanc(abs(dta[.,2])));
	_n1=2;  _n2=1;  _n3=2;
	if rows(beta_VR)==4;
		beta0 = beta_VR[1,1] | 0.0 | beta_VR[2,1] | zeros(1,1) | beta_VR[3:4,1];
	else;
		{beta0}=becinit(swd);
	endif;
	{xecp,llfecp,g,h,retcode}=swemmle(beta0,swd);
	retpstat[6,1]=retpstat[6,1]+retcode;

@ The NMP Model @
@	__title="Mixture of Normal Distributions Model: (Variable Switching Prob.)";@
@	?;      __title;        @
	_emecm=0;
/*
	swd = dta[.,1]~ones(nobs,3)~dta[.,2]~(dta[.,2].^2);
	_n1=1;  _n2=1;  _n3=3;
	if rows(beta_NM)==5;
		beta0 = beta_NM[1:3,1] | zeros(2,1) | beta_NM[4:5,1];
*/
	swd = dta[.,1]~ones(nobs,3)~(abs(dta[.,2])-meanc(abs(dta[.,2])));
	_n1=1;  _n2=1;  _n3=2;
	if rows(beta_NM)==5;
		beta0 = beta_NM[1:3,1] | zeros(1,1) | beta_NM[4:5,1];
	else;
		{beta0}=becinit(swd);
	endif;
	{xnmp,llfnmp,g,h,retcode}=swemmle(beta0,swd);
	retpstat[6,1]=retpstat[6,1]+retcode;

@ The SWP Model @
@	__title="Switching Regression Model: (Variable Switching Prob.) using rTSE and bTSEcs.";@
	_emecm=0;
/*
	swd = dta[.,1]~ones(nobs,1)~dta[.,2]~ones(nobs,1)~dta[.,2]~ones(nobs,1)~dta[.,2]~(dta[.,2].^2);
	_n1=2;  _n2=2;  _n3=3;
	if     (llfnmp > llfecp);
		beta0 = xnmp[1,1] | 0.0 | xnmp[2,1] | 0.0 | xnmp[3:7,1];
	else;
		beta0 = xecp[1:2,1] | xecp[1:2,1] | xecp[3:7,1];
	endif;
*/
	swd = dta[.,1]~ones(nobs,1)~dta[.,2]~ones(nobs,1)~dta[.,2]~ones(nobs,1)~(abs(dta[.,2])-meanc(abs(dta[.,2])));
	_n1=2;  _n2=2;  _n3=2;
	if     (llfnmp > llfecp);
		beta0 = xnmp[1,1] | 0.0 | xnmp[2,1] | 0.0 | xnmp[3:6,1];
	else;
		beta0 = xecp[1:2,1] | xecp[1:2,1] | xecp[3:6,1];
	endif;
	@load beta0 = d:\pitfalls\bTSEcs;@
	{xswp,llfswp,g,h,retcode}=swemmle(beta0,swd);
	retpstat[6,1]=retpstat[6,1]+retcode;

	llfec = llfec*nobs;	
	llfecp = llfecp*nobs;	
	llfnmp = llfnmp*nobs;	
	llfswp = llfswp*nobs;	
	retpstat[1,1] = 2*(llfswp -llfec);
	retpstat[2,1] = 2*(llfswp -llfecp);
	retpstat[3,1] = 2*(llfswp -llfnmp);

	if (retcode==5) or (rows(h)<(_n1+1));
		retpstat[4 5,1] = zeros(2,1);
	else;
		yjunk = 1 ~ -1;
		waldstat = yjunk*h[1 _n1+1,1 _n1+1]*yjunk' ;
		waldstat = (xswp[1,1] - xswp[_n1+1,1])^2 ./ waldstat;
		retpstat[4,1] = waldstat[1,1];
		waldstat = yjunk*h[2 _n1+2,2 _n1+2]*yjunk' ;
		waldstat = (xswp[2,1] - xswp[_n1+2,1])^2 ./ waldstat;
		retpstat[5,1] = waldstat[1,1];
	endif;

retp(retpstat);
endp;


/* WALDSHUF()
	This procedure accepts initial parameter estimates for the NM and SWP model as well as an N x 2 data matrix.
	It then estimates the SWP model and calculates some Wald test statistics.
		If the starting parameter estimates are missing, it will make its own starting estimates.
	Thereafter, it returns a 3 x 1 vector of test statistics, which are
		the Wald statistic testing B_S0 = B_C0
		the Wald statistic testing B_Sb = B_Cb
		return code for SWP for maxlik().
*/

proc WaldShuf(dta,beta_NM,beta_SWP);

local swd,nobs, beta0, yjunk, waldstat, retpstat,xecp,llfecp,xnmp,llfnmp,xswp,llfswp,g,h,retcode;

	nobs = rows(dta);
	retpstat=zeros(3,1);
	_max_GradProc=&swgrad;

@ The SWP Model @
@	__title="Switching Regression Model: (Variable Switching Prob.) using rTSE and bTSEcs.";@
	_emecm=0;
	swd = dta[.,1]~ones(nobs,1)~dta[.,2]~ones(nobs,1)~dta[.,2]~ones(nobs,1)~dta[.,2]~(dta[.,2].^2);
	_n1=2;  _n2=2;  _n3=3;

	beta0 = beta_NM[1,1] | 0.0 | beta_NM[2,1] | 0.0 | beta_NM[3,1] | zeros(2,1) | beta_NM[4:5,1];
	{xswp,llfswp,g,h,retcode}=swemmle(beta0,swd);
	retpstat[3,1]=retcode;

@	if rank(h[1 2 _n1+1 _n1+2, 1 2 _n1+1 _n1+2])<4;@
	if rows(h)<9;
		beta0 = beta_SWP[1,1] | 0.0 | beta_SWP[2,1] | 0.0 | beta_SWP[3,1] | zeros(2,1) | beta_SWP[4:5,1];
		{xswp,llfswp,g,h,retcode}=swemmle(beta0,swd);
		retpstat[3,1]=retcode;
	endif;
	
	if rows(h)<9;
		{xswp,llfswp,g,h,retcode}=swemmle(beta_SWP,swd);
		retpstat[3,1]=retcode;
	endif;
	
	if rows(h)<9;
		{beta0}=binitial(swd);
		{xswp,llfswp,g,h,retcode}=swemmle(beta0,swd);
		retpstat[3,1]=retcode;
	endif;
	
	if rows(h)<9;
		retpstat[1 2,1] = zeros(2,1);
	else;
		yjunk = 1 ~ -1;
		waldstat = yjunk*h[1 _n1+1,1 _n1+1]*yjunk' ;
		waldstat = (xswp[1,1] - xswp[_n1+1,1])^2 ./ waldstat;
		retpstat[1,1] = waldstat[1,1];
		waldstat = yjunk*h[2 _n1+2,2 _n1+2]*yjunk' ;
		waldstat = (xswp[2,1] - xswp[_n1+2,1])^2 ./ waldstat;
		retpstat[2,1] = waldstat[1,1];
	endif;

retp(retpstat);
endp;