clearg _swclass;
_swclass = "PROBIT";

proc (4)= swdescrp(xswp,h,swd);
local itertime, nobs, llfswp, g, retcode, fitted1, fitted2, fitted3, varf1, varf2, varofy, rsq1, rsq2, sigm1, sigm2, pxante, pxpost, rsqxante, rsqxpost, kcrit, pbull, pbear, mu, junk, restr, yjunk;

"SWDescrp - Descriptive Statistics for Switching Models";
itertime = date;
nobs=rows(swd);

@ Fitted Values and parameters @
fitted1 = swd[.,2:_n1+1]*xswp[1:_n1,1];
fitted2 = swd[.,2+_n1:_n1+_n2+1]*xswp[_n1+1:_n1+_n2,1];
fitted3 = swd[.,2+_n1+_n2:_n1+_n2+_n3+1]*xswp[_n1+_n2+1:_n1+_n2+_n3,1];
sigm1 = xswp[_n1+_n2+_n3+1,1];
sigm2 = xswp[_n1+_n2+_n3+2,1];

@ Select logit or probit weights (default is PROBIT) @
if upper(_swclass) $== "PROBIT";
	pxante = cdfn(-fitted3);
else;
	pxante = 1/(1+exp(fitted3));
endif;

@ Ex ante and ex post classification @
pxpost = pdfn((swd[.,1]-fitted1)./sigm1)./sigm1;
pxpost = pxpost.*pxante./(pxpost.*pxante + (1-pxante).*
	pdfn((swd[.,1]-fitted2)./sigm2)./sigm2 );

@ Probability of a 2 std dev. outlier @
kcrit = meanc(swd[.,1])+2*stdc(swd[.,1]);
pbull = 1 - cdfn((kcrit-fitted1)./sigm1).*pxante
	- cdfn((kcrit-fitted2)./sigm2).*(1-pxante);
kcrit = meanc(swd[.,1])-2*stdc(swd[.,1]);
pbear =     cdfn((kcrit-fitted1)./sigm1).*pxante
	+ cdfn((kcrit-fitted2)./sigm2).*(1-pxante);

@ R-Squared Measures  of fit @
varf1  = vcx(swd[.,1] - fitted1);	varf2  = vcx(swd[.,1] - fitted2);
varofy = vcx(swd[.,1]);
rsq1 = 1 - varf1 / varofy;		rsq2 = 1 - varf2 / varofy;
"r-sq of regime 1: ";; rsq1;;		"    r-sq of regime 2: ";; rsq2;
rsqxante = 1 - vcx(swd[.,1] - (pxante.*fitted1 + (1-pxante).*fitted2)) / varofy;
rsqxpost = 1 - vcx(swd[.,1] - (pxpost.*fitted1 + (1-pxpost).*fitted2)) / varofy;
"ex ante r-sq    : ";; rsqxante;;	"    ex post r-sq    : ";; rsqxpost;

@ Test significance of difference in expected values for all points in time. @
@ This version directly calculates the significance level. @
mu = fitted1 - fitted2;
junk=1;		restr = zeros(nobs,1);
do until junk > nobs;
    yjunk = swd[junk,2:_n1+1] ~ -swd[junk,2+_n1:1+_n1+_n2];
    restr[junk,1] = yjunk*h[1:_n1+_n2,1:_n1+_n2]*yjunk';
    junk = junk+1;
endo;
restr[.,1] = cdfchic( mu^2 ./ restr[.,1], 1);

"Total Calculation time = ";;  etstr(ethsec(itertime,date));
retp(fitted1~fitted2~fitted3, pxante~pxpost, pbull~pbear, mu~restr[.,1]);

endp;
