/*  This file contains a procedure swecmem() that takes a parameter vector and a data matrix, and then runs once through the EM estimation algorithm for switching regressions described by Hartley(1979), and returns an updated estimate of the parameter vector.  This is the special case where we restrict _n1=_n2, X1=X2, and beta1=beta2 (i.e. the error-correction or error-contamination case).  The conventions for the form of the data matrix and the parameter vector are same as for swli(), described in swcontam.g.   */

clearg _n1, _n2, _n3, nobs;

proc swecmem(binitial,dta);
local f1, f2, h, lamda, w, w1, w2, epsilon, beta, beta3, t1, t2, sigma1, sigma2, nparam, y, x, x3, y1, y2, res, wres, fit3;

beta = binitial[1:_n1,1];
beta3 = binitial[(_n1+1):(_n1+_n3),1];
sigma1 = binitial[(_n1+_n3+1),1];
sigma2 = binitial[(_n1+_n3+2),1];

y=dta[.,1];
x=dta[.,2:(_n1+1)];
x3=dta[.,(_n1+2):(_n1+_n3+1)];

res=y-x*beta;
fit3=x3*beta3;
f1=pdfn(res./sigma1)./sigma1;
f2=pdfn(res./sigma2)./sigma2;
lamda=cdfn(-fit3);
h=(lamda.*f1)+((1-lamda).*f2);
w1=lamda.*f1./h;
w2=1-w1;
w=w1./sigma1^2 + w2./sigma2^2;
epsilon=fit3+(f2-f1).*pdfn(-fit3)./h;

t1=sumc(w1);
t2=sumc(w2);
w=sqrt(w);
w1=sqrt(w1);
w2=sqrt(w2);

x=x.*w;
y=y.*w;
beta=y/x;
beta3=epsilon/x3;

res=(y-x*beta)./w;
sigma1=res.*w1;
sigma2=res.*w2;
sigma1=sqrt(sigma1'*sigma1/t1);
sigma2=sqrt(sigma2'*sigma2/t2);

retp(beta|beta3|sigma1|sigma2);

endp;
