/*  This proc computes starting values for error contamination models.

It is assumed that the first column of the dataset contains the dependent
(switching) variable. Note that the regressors need not be the same in all 3
regressions and a constant vector is not added.  Rows in the data set with
missing values are ignored using "packr".
  
The proc returns a vector of inital values for use in MAXLIK.  Positive values
of y are associated with equation 1.  */


proc 1=becinit(initd);
local b1, b3, sigma1, sigma2, x1, x3, y, spread, c, k;

initd = packr(initd);
y=initd[.,1];
x1=initd[.,2:(_n1+1)];
spread=ones(rows(initd),1);
c=0.001;

k=1;
do until k > 20;    k=k+1;
    b1=( y./sqrt(spread) ) / ( x1./sqrt(spread) );
    spread=c^2 + (y-x1*b1)^2;
endo;

x3=initd[.,(_n1+2):(_n1+_n3+1)];
b3=(spread-meanc(spread))/x3;
sigma1=meanc(sqrt( abs(y-x1*b1) )); sigma1=sigma1*sigma1;
sigma2=stdc(y-x1*b1);


retp(b1|b3|sigma1|sigma2);
endp;
